==> zio-aws-ec2_2.12-7.21.15.2-sources.jar: zio/aws/ec2/model/CreateLaunchTemplateRequest.scala <==
package zio.aws.ec2.model
import zio.aws.ec2.model.primitives.{VersionDescription, LaunchTemplateName}
import zio.ZIO
import zio.aws.core.{AwsError, BuilderHelper}
import zio.prelude.data.Optional
import scala.jdk.CollectionConverters._
final case class CreateLaunchTemplateRequest(
    dryRun: Optional[Boolean] = Optional.Absent,
    clientToken: Optional[String] = Optional.Absent,
    launchTemplateName: LaunchTemplateName,
    versionDescription: Optional[VersionDescription] = Optional.Absent,
    launchTemplateData: zio.aws.ec2.model.RequestLaunchTemplateData,
    tagSpecifications: Optional[Iterable[zio.aws.ec2.model.TagSpecification]] =
      Optional.Absent
) {
  def buildAwsValue()
      : software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest = {
    import CreateLaunchTemplateRequest.zioAwsBuilderHelper.BuilderOps
    software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
      .builder()
      .optionallyWith(dryRun.map(value => value: java.lang.Boolean))(_.dryRun)
      .optionallyWith(clientToken.map(value => value: java.lang.String))(
        _.clientToken
      )
      .launchTemplateName(
        LaunchTemplateName.unwrap(launchTemplateName): java.lang.String
      )
      .optionallyWith(
        versionDescription.map(value =>
          VersionDescription.unwrap(value): java.lang.String
        )
      )(_.versionDescription)
      .launchTemplateData(launchTemplateData.buildAwsValue())
      .optionallyWith(
        tagSpecifications.map(value =>
          value.map { item =>
            item.buildAwsValue()
          }.asJavaCollection
        )
      )(_.tagSpecifications)
      .build()
  }
  def asReadOnly: zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly =
    zio.aws.ec2.model.CreateLaunchTemplateRequest.wrap(buildAwsValue())
}
object CreateLaunchTemplateRequest {
  private lazy val zioAwsBuilderHelper: BuilderHelper[
    software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ] = BuilderHelper.apply
  trait ReadOnly {
    def asEditable: zio.aws.ec2.model.CreateLaunchTemplateRequest =
      zio.aws.ec2.model.CreateLaunchTemplateRequest(
        dryRun.map(value => value),
        clientToken.map(value => value),
        launchTemplateName,
        versionDescription.map(value => value),
        launchTemplateData.asEditable,
        tagSpecifications.map(value =>
          value.map { item =>
            item.asEditable
          }
        )
      )
    def dryRun: Optional[Boolean]
    def clientToken: Optional[String]
    def launchTemplateName: LaunchTemplateName
    def versionDescription: Optional[VersionDescription]
    def launchTemplateData: zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly
    def tagSpecifications
        : Optional[List[zio.aws.ec2.model.TagSpecification.ReadOnly]]
    def getDryRun: ZIO[Any, AwsError, Boolean] =
      AwsError.unwrapOptionField("dryRun", dryRun)
    def getClientToken: ZIO[Any, AwsError, String] =
      AwsError.unwrapOptionField("clientToken", clientToken)
    def getLaunchTemplateName: ZIO[Any, Nothing, LaunchTemplateName] =
      ZIO.succeed(launchTemplateName)
    def getVersionDescription: ZIO[Any, AwsError, VersionDescription] =
      AwsError.unwrapOptionField("versionDescription", versionDescription)
    def getLaunchTemplateData: ZIO[
      Any,
      Nothing,
      zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly
    ] = ZIO.succeed(launchTemplateData)
    def getTagSpecifications: ZIO[Any, AwsError, List[
      zio.aws.ec2.model.TagSpecification.ReadOnly
    ]] = AwsError.unwrapOptionField("tagSpecifications", tagSpecifications)
  }
  private final class Wrapper(
      impl: software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ) extends zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly {
    override val dryRun: Optional[Boolean] = zio.aws.core.internal
      .optionalFromNullable(impl.dryRun())
      .map(value => value: Boolean)
    override val clientToken: Optional[String] = zio.aws.core.internal
      .optionalFromNullable(impl.clientToken())
      .map(value => value: String)
    override val launchTemplateName: LaunchTemplateName =
      zio.aws.ec2.model.primitives.LaunchTemplateName(impl.launchTemplateName())
    override val versionDescription: Optional[VersionDescription] =
      zio.aws.core.internal
        .optionalFromNullable(impl.versionDescription())
        .map(value => zio.aws.ec2.model.primitives.VersionDescription(value))
    override val launchTemplateData
        : zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly =
      zio.aws.ec2.model.RequestLaunchTemplateData
        .wrap(impl.launchTemplateData())
    override val tagSpecifications
        : Optional[List[zio.aws.ec2.model.TagSpecification.ReadOnly]] =
      zio.aws.core.internal
        .optionalFromNullable(impl.tagSpecifications())
        .map(value =>
          value.asScala.map { item =>
            zio.aws.ec2.model.TagSpecification.wrap(item)
          }.toList
        )
  }
  def wrap(
      impl: software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ): zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly = new Wrapper(impl)
}

==> zio-aws-ec2_2.13-5.17.102.9-sources.jar: zio/aws/ec2/model/CreateLaunchTemplateRequest.scala <==
package zio.aws.ec2.model
import zio.aws.ec2.model.primitives.{VersionDescription, LaunchTemplateName}
import zio.ZIO
import zio.aws.core.{AwsError, BuilderHelper}
import scala.jdk.CollectionConverters._
final case class CreateLaunchTemplateRequest(
    clientToken: Option[String] = None,
    launchTemplateName: LaunchTemplateName,
    versionDescription: Option[VersionDescription] = None,
    launchTemplateData: zio.aws.ec2.model.RequestLaunchTemplateData,
    tagSpecifications: Option[Iterable[zio.aws.ec2.model.TagSpecification]] =
      None
) {
  def buildAwsValue()
      : software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest = {
    import CreateLaunchTemplateRequest.zioAwsBuilderHelper.BuilderOps
    software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
      .builder()
      .optionallyWith(clientToken.map(value => value: java.lang.String))(
        _.clientToken
      )
      .launchTemplateName(
        LaunchTemplateName.unwrap(launchTemplateName): java.lang.String
      )
      .optionallyWith(
        versionDescription.map(value =>
          VersionDescription.unwrap(value): java.lang.String
        )
      )(_.versionDescription)
      .launchTemplateData(launchTemplateData.buildAwsValue())
      .optionallyWith(
        tagSpecifications.map(value =>
          value.map { item =>
            item.buildAwsValue()
          }.asJavaCollection
        )
      )(_.tagSpecifications)
      .build()
  }
  def asReadOnly: zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly =
    zio.aws.ec2.model.CreateLaunchTemplateRequest.wrap(buildAwsValue())
}
object CreateLaunchTemplateRequest {
  private lazy val zioAwsBuilderHelper: BuilderHelper[
    software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ] = BuilderHelper.apply
  trait ReadOnly {
    def asEditable: zio.aws.ec2.model.CreateLaunchTemplateRequest =
      zio.aws.ec2.model.CreateLaunchTemplateRequest(
        clientToken.map(value => value),
        launchTemplateName,
        versionDescription.map(value => value),
        launchTemplateData.asEditable,
        tagSpecifications.map(value =>
          value.map { item =>
            item.asEditable
          }
        )
      )
    def clientToken: Option[String]
    def launchTemplateName: LaunchTemplateName
    def versionDescription: Option[VersionDescription]
    def launchTemplateData: zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly
    def tagSpecifications
        : Option[List[zio.aws.ec2.model.TagSpecification.ReadOnly]]
    def getClientToken: ZIO[Any, AwsError, String] =
      AwsError.unwrapOptionField("clientToken", clientToken)
    def getLaunchTemplateName: ZIO[Any, Nothing, LaunchTemplateName] =
      ZIO.succeed(launchTemplateName)
    def getVersionDescription: ZIO[Any, AwsError, VersionDescription] =
      AwsError.unwrapOptionField("versionDescription", versionDescription)
    def getLaunchTemplateData: ZIO[
      Any,
      Nothing,
      zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly
    ] = ZIO.succeed(launchTemplateData)
    def getTagSpecifications: ZIO[Any, AwsError, List[
      zio.aws.ec2.model.TagSpecification.ReadOnly
    ]] = AwsError.unwrapOptionField("tagSpecifications", tagSpecifications)
  }
  private final class Wrapper(
      impl: software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ) extends zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly {
    override val clientToken: Option[String] =
      scala.Option(impl.clientToken()).map(value => value: String)
    override val launchTemplateName: LaunchTemplateName =
      zio.aws.ec2.model.primitives.LaunchTemplateName(impl.launchTemplateName())
    override val versionDescription: Option[VersionDescription] = scala
      .Option(impl.versionDescription())
      .map(value => zio.aws.ec2.model.primitives.VersionDescription(value))
    override val launchTemplateData
        : zio.aws.ec2.model.RequestLaunchTemplateData.ReadOnly =
      zio.aws.ec2.model.RequestLaunchTemplateData
        .wrap(impl.launchTemplateData())
    override val tagSpecifications
        : Option[List[zio.aws.ec2.model.TagSpecification.ReadOnly]] = scala
      .Option(impl.tagSpecifications())
      .map(value =>
        value.asScala.map { item =>
          zio.aws.ec2.model.TagSpecification.wrap(item)
        }.toList
      )
  }
  def wrap(
      impl: software.amazon.awssdk.services.ec2.model.CreateLaunchTemplateRequest
  ): zio.aws.ec2.model.CreateLaunchTemplateRequest.ReadOnly = new Wrapper(impl)
}
